==> cl_editorial-1.0.0-beta3.tar.gz: cl_editorial/src/NoThemeComponentManager.php <==
<?php

namespace Drupal\cl_editorial;

use Drupal\cl_components\Component\ComponentMetadata;
use Drupal\cl_components\ComponentPluginManager;
use Drupal\cl_components\ExtensionType;
use Drupal\cl_components\Plugin\Component;

/**
 * Decorates the component plugin manager to add more features.
 */
class NoThemeComponentManager {

  /**
   * Constructs a NoThemeComponentManager.
   *
   * @param \Drupal\cl_components\ComponentPluginManager $pluginManager
   *   The component plugin manager.
   */
  public function __construct(protected readonly ComponentPluginManager $pluginManager) {
  }

  /**
   * Returns the components matching the filters.
   *
   * @param array $allowed
   *   The allowed components.
   * @param array $forbidden
   *   The forbidden components.
   * @param array $types
   *   The allowed component types.
   * @param array $statuses
   *   The allowed component statuses.
   *
   * @return \Drupal\cl_components\Plugin\Component[]
   *   The list of allowed values.
   *
   * @internal
   */
  public function getFilteredComponentTypes(
    array $allowed = [],
    array $forbidden = [],
    array $types = [ComponentMetadata::COMPONENT_TYPE_ORGANISM],
    array $statuses = [
      ComponentMetadata::COMPONENT_STATUS_READY,
      ComponentMetadata::COMPONENT_STATUS_BETA,
    ]
  ): array {
    $plugin_ids = array_keys($this->getDefinitionsWithoutDupes());
    $components = array_values(array_map(
      [$this->pluginManager, 'createInstanceAndCatch'],
      $plugin_ids
    ));
    $filter_components = static function (Component $component) use ($types, $statuses, $allowed, $forbidden) {
      $metadata = $component->getMetadata();
      $machine_name = $metadata->getMachineName();
      if (!in_array($metadata->getComponentType(), $types, TRUE)) {
        return FALSE;
      }
      if (!in_array($metadata->getStatus(), $statuses, TRUE)) {
        return FALSE;
      }
      // If there are allowed components and this is not one of them, continue.
      if (!empty($allowed) && !in_array($machine_name, $allowed, TRUE)) {
        return FALSE;
      }
      // If this is a forbidden component, continue.
      if (in_array($machine_name, $forbidden, TRUE)) {
        return FALSE;
      }
      return TRUE;
    };
    $filtered_components = array_reduce(
      array_filter($components, $filter_components),
      static fn(array $carry, Component $component) => [
        ...$carry,
        $component->getPluginId() => $component,
      ],
      []
    );
    ksort($filtered_components);
    return $filtered_components;
  }

  /**
   * Gets the plugin definitions after resolving plugin overrides.
   *
   * @return array[]
   *   The definition arrays.
   *
   * @internal
   */
  protected function getDefinitionsWithoutDupes(): array {
    $definitions = $this->getDefinitions();
    $deduped_definitions = array_reduce(
      $definitions,
      function (array $carry, array $definition) use ($definitions) {
        $new_definition = $definition['extension_type'] === ExtensionType::Theme
          ? $this->guessThemeOverrides($definition['machineName'], $definitions)
          : $this->filterModuleReplacements($definition['machineName'], $definitions);
        return [
          ...$carry,
          $new_definition['machineName'] => $new_definition,
        ];
      },
      []
    );
    return array_reduce(
      $deduped_definitions,
      static fn(array $carry, array $definition) => [
        ...$carry,
        $definition['id'] => $definition,
      ],
      []
    );
  }

  /**
   * Gets a component for rendering without filter considerations.
   *
   * @param string $machine_name
   *   The machine name.
   *
   * @return \Drupal\cl_components\Plugin\Component
   *   The component.
   *
   * @internal
   */
  public function findWithoutThemeFilter(string $machine_name): Component {
    $definitions = array_filter(
      $this->getDefinitionsWithoutDupes(),
      static fn(array $definition) => $definition['machineName'] === $machine_name
    );
    $definition = reset($definitions);
    return $this->createInstance($definition['id'] ?? '');
  }

  /**
   * Proxies the createInstance method.
   */
  public function createInstance($plugin_id, array $configuration = []) {
    return $this->pluginManager->createInstance($plugin_id, $configuration);
  }

  /**
   * Proxies the getDefinitions method.
   */
  public function getDefinitions() {
    return $this->pluginManager->getDefinitions();
  }

  /**
   * Proxies the find method.
   */
  public function find(string $machine_name): Component {
    return $this->pluginManager->find($machine_name);
  }

  /**
   * Guess theme override.
   *
   * Let's be said that we don't have a reliable way to get a component type
   * representative amongst different theme overrides. However, we can guess
   * that themes with longer theme inheritance lineage will be more specific.
   */
  private function guessThemeOverrides(string $machine_name, array $all_definitions) {
    $theme_handler = \Drupal::service('theme_handler');
    $candidates = array_filter(
      $all_definitions,
      static fn(array $definition) => $definition['extension_type'] === ExtensionType::Theme
        && $definition['machineName'] === $machine_name
    );
    $get_weight = static fn(string $theme_name) => count($theme_handler->getBaseThemes(
      $theme_handler->listInfo(),
      $theme_name
    ));
    $sort_by_lineage_length = static function (array $definition_a, array $definition_b) use ($theme_handler, $get_weight) {
      $a_weight = $get_weight($definition_a['provider']);
      $b_weight = $get_weight($definition_b['provider']);
      return $b_weight <=> $a_weight;
    };
    uasort($candidates, $sort_by_lineage_length);
    $definition = reset($candidates);
    return $definition ?: NULL;
  }

  /**
   * Negotiate the component from the list of candidates for a module.
   *
   * @param string $machine_name
   *   The machine name.
   * @param array[] $all_definitions
   *   The candidate definitions.
   *
   * @return array|null
   *   The negotiated plugin definition, or NULL if none found.
   */
  private function filterModuleReplacements(string $machine_name, array $all_definitions): ?array {
    $module_list = \Drupal::service('extension.list.module')->getList();
    if (!$module_list) {
      return NULL;
    }
    $candidates = array_filter(
      $all_definitions,
      static fn(array $definition) => $definition['machineName'] === $machine_name
    );
    // Now filter out all module based components that are replaced.
    $replaced_components = array_unique(array_values(array_filter(array_map(
      static function (array $definition): ?string {
        if ($definition['extension_type'] !== ExtensionType::Module) {
          return NULL;
        }
        return $definition['replaces'] ?? NULL;
      },
      $candidates
    ))));
    $candidates = array_filter(
      $candidates,
      static fn(array $definition) => $definition['extension_type'] === ExtensionType::Module
        && !(in_array($definition['id'], $replaced_components, TRUE))
    );
    $sort_by_module_weight = static function (array $definition_a, array $definition_b) use ($module_list) {
      $a_weight = $module_list[$definition_a['provider'] ?? '']?->weight ?? 999;
      $b_weight = $theme_weights[$definition_b['provider'] ?? '']?->weight ?? 999;
      return $a_weight <=> $b_weight;
    };
    uasort($candidates, $sort_by_module_weight);
    $definition = reset($candidates);
    return $definition ?: NULL;
  }

}

==> cl_editorial-2.0.0-beta3.tar.gz: cl_editorial/src/NoThemeComponentManager.php <==
<?php

namespace Drupal\cl_editorial;

use Drupal\Core\Extension\ExtensionLifecycle;
use Drupal\sdc\Component\ComponentMetadata;
use Drupal\sdc\ComponentPluginManager;
use Drupal\sdc\Exception\ComponentNotFoundException;
use Drupal\sdc\ExtensionType;
use Drupal\sdc\Plugin\Component;

/**
 * Decorates the component plugin manager to add more features.
 */
class NoThemeComponentManager {

  /**
   * Constructs a NoThemeComponentManager.
   *
   * @param \Drupal\sdc\ComponentPluginManager $pluginManager
   *   The component plugin manager.
   */
  public function __construct(protected readonly ComponentPluginManager $pluginManager) {
  }

  /**
   * Returns the components matching the filters.
   *
   * @param array $allowed
   *   The allowed components.
   * @param array $forbidden
   *   The forbidden components.
   * @param array $statuses
   *   The allowed component statuses.
   *
   * @return \Drupal\sdc\Plugin\Component[]
   *   The list of allowed values.
   *
   * @internal
   */
  public function getFilteredComponents(
    array $allowed = [],
    array $forbidden = [],
    array $statuses = [
      ExtensionLifecycle::STABLE,
      ExtensionLifecycle::EXPERIMENTAL,
    ]
  ): array {
    $plugin_ids = array_keys($this->getDefinitionsWithoutDupes());
    $components = array_values(array_filter(array_map(
      [$this, 'createInstanceAndCatch'],
      $plugin_ids
    )));
    $filter_components = static function (Component $component) use ($statuses, $allowed, $forbidden) {
      $metadata = $component->metadata;
      $machine_name = $metadata->machineName;
      if (!in_array($metadata->status, $statuses, TRUE)) {
        return FALSE;
      }
      // If there are allowed components and this is not one of them, continue.
      if (!empty($allowed) && !in_array($machine_name, $allowed, TRUE)) {
        return FALSE;
      }
      // If this is a forbidden component, continue.
      if (in_array($machine_name, $forbidden, TRUE)) {
        return FALSE;
      }
      return TRUE;
    };
    $filtered_components = array_reduce(
      array_filter($components, $filter_components),
      static fn(array $carry, Component $component) => [
        ...$carry,
        $component->getPluginId() => $component,
      ],
      []
    );
    ksort($filtered_components);
    return $filtered_components;
  }

  /**
   * Gets the plugin definitions after resolving plugin overrides.
   *
   * @return array[]
   *   The definition arrays.
   *
   * @internal
   */
  protected function getDefinitionsWithoutDupes(): array {
    $definitions = $this->getDefinitions();
    $deduped_definitions = array_reduce(
      $definitions,
      function (array $carry, array $definition) use ($definitions) {
        $new_definition = $definition['extension_type'] === ExtensionType::Theme
          ? $this->guessThemeOverrides($definition['machineName'], $definitions)
          : $this->filterModuleReplacements($definition['machineName'], $definitions);
        return [
          ...$carry,
          $new_definition['machineName'] => $new_definition,
        ];
      },
      []
    );
    return array_reduce(
      $deduped_definitions,
      static fn(array $carry, array $definition) => [
        ...$carry,
        $definition['id'] => $definition,
      ],
      []
    );
  }

  /**
   * Gets a component for rendering without filter considerations.
   *
   * @param string $plugin_id
   *   The plugin ID.
   *
   * @return \Drupal\sdc\Plugin\Component
   *   The component.
   *
   * @throws \Drupal\sdc\Exception\ComponentNotFoundException
   *
   * @internal
   */
  public function findWithoutThemeFilter(string $plugin_id): Component {
    $definitions = array_filter(
      $this->getDefinitionsWithoutDupes(),
      static fn(array $definition) => $definition['id'] === $plugin_id
    );
    $definition = reset($definitions);
    return $this->pluginManager->createInstance($definition['id'] ?? '');
  }

  /**
   * Proxies the createInstance method.
   */
  public function createInstanceAndCatch($plugin_id, array $configuration = []) {
    try {
      return $this->pluginManager->createInstance($plugin_id, $configuration);
    }
    catch (ComponentNotFoundException $e) {
      return NULL;
    }
  }

  /**
   * Proxies the getDefinitions method.
   */
  public function getDefinitions(): ?array {
    return $this->pluginManager->getDefinitions();
  }

  /**
   * Proxies the find method.
   *
   * @throws \Drupal\sdc\Exception\ComponentNotFoundException
   */
  public function find(string $machine_name): Component {
    return $this->pluginManager->find($machine_name);
  }

  /**
   * Guess theme override.
   *
   * Let's be said that we don't have a reliable way to get a component
   * representative amongst different theme overrides. However, we can guess
   * that themes with longer theme inheritance lineage will be more specific.
   */
  private function guessThemeOverrides(string $machine_name, array $all_definitions) {
    $theme_handler = \Drupal::service('theme_handler');
    $candidates = array_filter(
      $all_definitions,
      static fn(array $definition) => $definition['extension_type'] === ExtensionType::Theme
        && $definition['machineName'] === $machine_name
    );
    $get_weight = static fn(string $theme_name) => count($theme_handler->getBaseThemes(
      $theme_handler->listInfo(),
      $theme_name
    ));
    $sort_by_lineage_length = static function (array $definition_a, array $definition_b) use ($theme_handler, $get_weight) {
      $a_weight = $get_weight($definition_a['provider']);
      $b_weight = $get_weight($definition_b['provider']);
      return $b_weight <=> $a_weight;
    };
    uasort($candidates, $sort_by_lineage_length);
    $definition = reset($candidates);
    return $definition ?: NULL;
  }

  /**
   * Negotiate the component from the list of candidates for a module.
   *
   * @param string $machine_name
   *   The machine name.
   * @param array[] $all_definitions
   *   The candidate definitions.
   *
   * @return array|null
   *   The negotiated plugin definition, or NULL if none found.
   */
  private function filterModuleReplacements(string $machine_name, array $all_definitions): ?array {
    $module_list = \Drupal::service('extension.list.module')->getList();
    if (!$module_list) {
      return NULL;
    }
    $candidates = array_filter(
      $all_definitions,
      static fn(array $definition) => $definition['machineName'] === $machine_name
    );
    // Now filter out all module based components that are replaced.
    $replaced_components = array_unique(array_values(array_filter(array_map(
      static function (array $definition): ?string {
        if ($definition['extension_type'] !== ExtensionType::Module) {
          return NULL;
        }
        return $definition['replaces'] ?? NULL;
      },
      $candidates
    ))));
    $candidates = array_filter(
      $candidates,
      static fn(array $definition) => $definition['extension_type'] === ExtensionType::Module
        && !(in_array($definition['id'], $replaced_components, TRUE))
    );
    $sort_by_module_weight = static function (array $definition_a, array $definition_b) use ($module_list) {
      $a_weight = $module_list[$definition_a['provider'] ?? '']?->weight ?? 999;
      $b_weight = $theme_weights[$definition_b['provider'] ?? '']?->weight ?? 999;
      return $a_weight <=> $b_weight;
    };
    uasort($candidates, $sort_by_module_weight);
    $definition = reset($candidates);
    return $definition ?: NULL;
  }

}
